@@ -16,6 +16,7 @@
 #include <pyublas/numpy.hpp>
 #include <boost/python/converter/implicit.hpp>
 #include <boost/python/converter/registry.hpp>
+#include <boost/format.hpp>
 
 #include <iostream>
 #include <boost/numeric/ublas/io.hpp>
@@ -32,6 +33,18 @@ namespace ublas = boost::numeric::ublas;
 
 namespace
 {
+  bool trace_conversion = false;
+
+  template<class T>
+  typename numpy_vector<T>::size_type unstrided_size(const numpy_vector<T> &v)
+  { return v.size(); }
+
+  template<class T>
+  typename numpy_strided_vector<T>::size_type strided_size(const numpy_strided_vector<T> &v)
+  { return v.size(); }
+
+
+
   template <class TargetType>
   struct array_converter_base
   {
@@ -39,7 +52,7 @@ namespace
     typedef TargetType target_type;
 
     static void construct(
-        PyObject* obj, 
+        PyObject* obj,
         py::converter::rvalue_from_python_stage1_data* data)
     {
       void* storage = ((py::converter::rvalue_from_python_storage<target_type>*)data)->storage.bytes;
@@ -51,7 +64,7 @@ namespace
     }
 
     static void construct_invalid_ok(
-        PyObject* obj, 
+        PyObject* obj,
         py::converter::rvalue_from_python_stage1_data* data)
     {
       typedef invalid_ok<target_type> inv_ok;
@@ -65,7 +78,7 @@ namespace
 
     template <class RealTgtType>
     static void construct_indirect(
-        PyObject* obj, 
+        PyObject* obj,
         py::converter::rvalue_from_python_stage1_data* data)
     {
       void* storage = ((py::converter::rvalue_from_python_storage<target_type>*)data)->storage.bytes;
@@ -106,7 +119,7 @@ namespace
 
     public:
       static void construct_strided(
-          PyObject* obj, 
+          PyObject* obj,
           py::converter::rvalue_from_python_stage1_data* data)
       {
         typedef numpy_strided_vector<typename super::value_type> strided_vec;
@@ -123,10 +136,25 @@ namespace
       static void *check(PyObject* obj)
       {
         if (!PyArray_Check(obj))
+        {
+          if (trace_conversion)
+            std::cerr
+              << boost::format("obj %1% rejected as vec: not a numpy array") % obj
+              << std::endl;
           return 0;
+        }
 
         if (!is_storage_compatible<typename super::value_type>(obj))
+        {
+          if (trace_conversion)
+            std::cerr
+              << boost::format("obj %1% rejected as vec: not storage-compatible with %2%")
+                  % obj % typeid(typename super::value_type).name()
+              << std::endl;
+
           return 0;
+        }
+
 
         return obj;
       }
@@ -135,13 +163,14 @@ namespace
       {
         if (obj == Py_None)
           return obj;
+
         return check(obj);
       }
 
       // needs to be overridden to copy strided version
       template <class RealTgtType>
       static void construct_indirect(
-          PyObject* obj, 
+          PyObject* obj,
           py::converter::rvalue_from_python_stage1_data* data)
       {
         void* storage = ((py::converter::rvalue_from_python_storage<
@@ -171,30 +200,82 @@ namespace
       static void *check(PyObject* obj)
       {
         if (!PyArray_Check(obj))
+        {
+          if (trace_conversion)
+            std::cerr
+              << boost::format("obj %1% rejected as mat: not a numpy array") % obj
+              << std::endl;
           return 0;
+        }
+
         if (!is_storage_compatible<typename super::value_type>(obj))
+        {
+          if (trace_conversion)
+            std::cerr
+              << boost::format("obj %1% rejected as mat: not storage-compatible with %2%")
+                  % obj % typeid(typename super::value_type).name()
+              << std::endl;
+
           return 0;
+        }
+
         if (PyArray_NDIM(obj) != 2)
+        {
+          if (trace_conversion)
+            std::cerr
+              << boost::format("obj %1% rejected as mat: not 2-dimensional") % obj
+              << std::endl;
           return 0;
+        }
 
         if (PyArray_STRIDE(obj, 1) == PyArray_ITEMSIZE(obj))
         {
           // row-major
           if (!is_row_major(typename MatrixType::orientation_category()))
+          {
+            if (trace_conversion)
+              std::cerr
+                << boost::format("obj %1% rejected as mat: not row-major") % obj
+                << std::endl;
             return 0;
+          }
+
           if (!PyArray_CHKFLAGS(obj, NPY_C_CONTIGUOUS))
+          {
+            if (trace_conversion)
+              std::cerr
+                << boost::format("obj %1% rejected as row-major mat: not C-contiguous") % obj
+                << std::endl;
             return 0;
+          }
         }
         else if (PyArray_STRIDE(obj, 0) == PyArray_ITEMSIZE(obj))
         {
           if (is_row_major(typename MatrixType::orientation_category()))
+          {
+            if (trace_conversion)
+              std::cerr
+                << boost::format("obj %1% rejected as mat: not column-major") % obj
+                << std::endl;
             return 0;
+          }
+
           if (!PyArray_CHKFLAGS(obj, NPY_F_CONTIGUOUS))
+          {
+            if (trace_conversion)
+              std::cerr
+                << boost::format("obj %1% rejected as column-major mat: not Fortran-contiguous") % obj
+                << std::endl;
             return 0;
+          }
         }
         else
         {
           // no dim has stride == 1
+          if (trace_conversion)
+            std::cerr
+              << boost::format("obj %1% rejected as mat: no unit-size stride") % obj
+              << std::endl;
           return 0;
         }
 
@@ -282,22 +363,22 @@ namespace
   // array scalars ------------------------------------------------------------
   template <class T>
   const PyTypeObject *get_array_scalar_typeobj()
-  { 
-    return (PyTypeObject *) PyArray_TypeObjectFromType(get_typenum(T())); 
+  {
+    return (PyTypeObject *) PyArray_TypeObjectFromType(get_typenum(T()));
   }
 
   template <class T>
   void *check_array_scalar(PyObject *obj)
-  { 
+  {
     if (obj->ob_type == get_array_scalar_typeobj<T>())
-      return obj; 
+      return obj;
     else
       return 0;
   }
 
   template <class T>
   static void convert_array_scalar(
-      PyObject* obj, 
+      PyObject* obj,
       py::converter::rvalue_from_python_stage1_data* data)
   {
     void* storage = ((py::converter::rvalue_from_python_storage<T>*)data)->storage.bytes;
@@ -327,14 +408,17 @@ namespace
         );
 
     // conversion of arrays
+    typedef numpy_array<T> ary;
     typedef numpy_vector<T> vec;
     typedef numpy_matrix<T, ublas::row_major> rm_mat;
     typedef numpy_matrix<T, ublas::column_major> cm_mat;
 
+    typedef vector_converter<ary> ary_converter;
     typedef vector_converter<vec> vec_converter;
     typedef matrix_converter<cm_mat> cm_mat_converter;
     typedef matrix_converter<rm_mat> rm_mat_converter;
 
+    register_array_converter<ary_converter>();
     register_vector_converter<vec_converter>();
     register_array_converter<cm_mat_converter>();
     register_array_converter<rm_mat_converter>();
@@ -345,10 +429,26 @@ namespace
       <vec_converter, ublas::bounded_vector<T, 2> >();
     register_indirect_array_converter
       <vec_converter, ublas::bounded_vector<T, 3> >();
+    register_indirect_array_converter
+      <vec_converter, ublas::bounded_vector<T, 4> >();
     register_indirect_array_converter
       <rm_mat_converter, ublas::matrix<T, ublas::row_major> >();
     register_indirect_array_converter
       <cm_mat_converter, ublas::matrix<T, ublas::column_major> >();
+
+    py::to_python_converter<
+        numpy_strided_vector<T>,
+        typename vector_converter<numpy_strided_vector<T> >::to_python>();
+
+    py::def("unstrided_size", unstrided_size<T>);
+    py::def("strided_size", strided_size<T>);
+  }
+
+
+
+  void set_trace(bool t)
+  {
+    trace_conversion = t;
   }
 }
 
@@ -357,9 +457,9 @@ namespace
 
 void pyublas_expose_converters()
 {
-  import_array();
-
+  expose_converters<bool>();
   expose_converters<npy_byte>();
+  expose_converters<npy_ubyte>();
   expose_converters<npy_short>();
   expose_converters<npy_ushort>();
   expose_converters<npy_int>();
@@ -372,4 +472,10 @@ void pyublas_expose_converters()
   expose_converters<npy_double>();
   expose_converters<std::complex<float> >();
   expose_converters<std::complex<double> >();
+#if HAVE_LONG_DOUBLE            // defined in pyconfig.h
+  expose_converters<npy_longdouble>();
+  expose_converters<std::complex<long double> >();
+#endif
+
+  py::def("set_trace", set_trace);
 }
